class type parameter =
object
	val name : string
	val gtype : grammar
	method get_name : string
	method get_type : grammar
	method get_shifts : (grammar * string) list
	method get_throughs : grammar list
	method set_gmodule : gmodule -> unit
	method is_index : bool
	method induction_pattern : grammar -> string list
	method rewrite_induction : property -> grammar -> string
	method to_string : unit -> string
end

and constructor =
object
	method get_name : string
	method get_index_name : string
	method get_params : parameter list
	method get_throughs : grammar list
	method is_index : bool
	method set_gmodule : gmodule -> unit
	method induction_pattern : grammar -> string list
	method rewrite_induction : property -> grammar -> string
	method fun_equation_gen : fun_definition -> string list
	method to_string : unit -> string
end

and grammar =
object
	method get_name : string
	method get_throughs : grammar list
	method get_constrs : constructor list
	method get_index_constructor_name : string option
	method has_index : bool
	method set_gmodule : gmodule -> unit
	method throughs_expand : bool
	method throughs_init : unit
	method by_induction : string -> string
	method rewrite_induction : property -> string list
	method fun_equations_gen : fun_definition -> string list
	method property_gen : property -> string -> node -> string list * string list * string list * (string * (string list * string list))
	method to_string : unit -> string
end

and node =
object
	method get_grams : grammar list
	method set_gmodule : gmodule -> unit
	method throughs_expand : bool
	method throughs_init : unit
	method syntax_gen : terms_definition -> string list
	method function_gen : fun_definition -> string list
	method property_gen : property -> string list * (string * (string list * string list)) list
	method to_string : unit -> string
end

and gmodule =
object
	method coq_output : string list
	method is_gmodule_gram : string -> bool
	method get_gram : string -> grammar
	method get_all_grams : grammar list
	method get_indexed_grams : grammar list
	method get_indexed_grams_through : grammar -> grammar list
	method property_gen : property -> string list * (string * (string list * string list)) list
	method to_string : unit -> string
end

and virtual terms_definition =
object
  val gmod : gmodule
	
	method set_target : grammar -> unit
	method get_target : grammar

	method virtual title : string
	method virtual index_type : string
	method coq_predefs : string list
	method aux_defs : string list

	method decorate_type_name : string -> string
	method decorate_constr_name : string -> string

	method virtual shift_terms_gen : (grammar * string) -> string
	method parameter_terms_gen : parameter -> string
	method constructor_terms_gen : constructor -> string
	method grammar_terms_gen : string -> grammar -> string list
end

and virtual fun_definition =
object
	val gmod : gmodule
	
	method virtual title : string
	method coq_predefs : string list
	method aux_defs : string list
	method virtual fun_params : (string * string) list

	method set_source : grammar -> unit
	method set_target : grammar -> unit
	method get_source : grammar
	method get_target : grammar
			
	method is_valid : bool
	method decorate_type_name : string -> string
	method decorate_constr_name : string -> string
	
	method fun_name : string
	method virtual fun_name_type_name : string -> string
	method virtual return_type : string
	
	method virtual get_source_grams : grammar option list
	method virtual get_target_grams : grammar list -> grammar list
	
	method is_self_rec : parameter -> bool
	method is_back_rec : parameter -> bool
	method is_throughs_rec : parameter -> bool
	method is_module_rec : parameter -> bool

	(** generation functions *)	
	method virtual shift_fun_gen : (grammar * string) -> string
	method virtual parameter_function_gen : parameter -> string
	method virtual index_constructor_rhs_function_gen : constructor -> string list
	method constructor_rhs_function_gen : constructor -> string list
	method rhs_function_gen : constructor -> string list
	method constructor_function_gen : constructor -> string list
	method constructor_function_equation_gen : constructor -> string list
	method grammar_function_gen : string -> grammar -> string list
end

and property =
object
	val gmod : gmodule
	
	method aux_defs : string list
	
	method set_node : node -> unit
	method set_source : grammar -> unit
	method set_target : grammar -> unit
	method get_node : node
	method get_source : grammar
	method get_target : grammar
	
	method get_source_grams : grammar list
	method get_target_grams : grammar list -> grammar list
	
	method is_valid_induction : grammar -> grammar -> bool
	
	method lemma_name_type_name : string -> string
	method lemma_name : string
	method lemma_header : string list
	method lemma_proof : string list
	method tactic_template : string * (string list * string list)
	
	method get_index_tac : string
	
	method subst : (string * string) list

	method generate : string list * (string * (string list * string list)) list
end

and property_pack =
object
	val gmod : gmodule

	method properties : property list
	method tactic_name : string option
	method get_title : string
	method generate : string list
end

and property_chapter =
object
	val gmod : gmodule

	method property_packs : property_pack list
	method packs_tactic_names : string list
	method before_defs : string list
	method after_defs : string list
	method get_title : string
	method generate : string list
end
