%{
	open Interfaces
	
  open Parameter
  open Constructor
  open Grammar
  open Node
  open Gmodule
	
	let gmod = new gmodule_impl "" [] [] []
%}

%token <string> ID
%token <int> INTEGER
%token <string list> VERBATIM_HEADER
%token <string list> VERBATIM_FOOTER

%token EOL

%token MODULE BEGIN END VERBATIM
%token INDUCTIVE TYPE WITH INDEX BIND IN

%token DOT LPARSTAR STARRPAR COLON BAR LPAR RPAR
%token COMMA LSQUARE RSQUARE ASSIGN
%token PLUS MINUS MULT

%start gmod
%type <Interfaces.gmodule> gmod

%%

gmod:
  MODULE ID DOT nodes END ID DOT
  { new gmodule_impl $2 [] $4 [] }
| MODULE ID DOT VERBATIM_HEADER nodes END ID DOT
  { new gmodule_impl $2 $4 $5 [] }
| MODULE ID DOT nodes VERBATIM_FOOTER END ID DOT
  { new gmodule_impl $2 [] $4 $5 }
| MODULE ID DOT VERBATIM_HEADER nodes VERBATIM_FOOTER END ID DOT
  { new gmodule_impl $2 $4 $5 $6 }

nodes:
  node
  { [ $1 ] }
| node nodes
  { $1 :: $2 }

node:
  INDUCTIVE grams DOT
  { new node_impl gmod $2 }

grams:
  ID COLON TYPE ASSIGN cstrs
  { new grammar_impl gmod $1 $5 :: [] }
| ID COLON TYPE ASSIGN cstrs WITH grams
  { new grammar_impl gmod $1 $5 :: $7 }

cstrs:
  cstr
  { [ $1 ] }
| cstr cstrs
  { $1 :: $2 }

cstr:
  BAR ID params
  { new constructor_impl gmod $2 $3 }
| BAR ID
  { new constructor_impl gmod $2 [] }

params:
  param
  { [ $1 ] }
| param params
  { $1 :: $2 }

param:
  LPAR LPARSTAR INDEX STARRPAR ID COLON ID RPAR
  { new index_parameter_impl gmod $5 (new external_impl "nat") }
| LPAR shift ID COLON ID RPAR
  { new parameter_impl gmod (List.map (fun (x, y) -> (y, x)) $2) $3 (new external_impl $5) }
| LPAR ID COLON ID RPAR
  { new parameter_impl gmod [] $2 (new external_impl $4) }

shift:
  LPARSTAR BIND shifts IN STARRPAR
  { $3 }

shifts:
  oneshift
  { [ $1 ] }
| oneshift COMMA shifts
  { $1 :: $3 }

oneshift:
  ID
  { ("1", new external_impl $1) }
| LSQUARE exp ID RSQUARE
  { ($2, new external_impl $3) }

exp:
	INTEGER
	{ string_of_int $1 }
| ID
	{ $1 }
| exp PLUS exp
	{ $1 ^ " + " ^ $3 }
| exp MINUS exp
	{ $1 ^ " - " ^ $3 }
| exp MULT exp
	{ $1 ^ " * " ^ $3 }
