module Xcoq = struct
	
	open Xlist
	open Xstring
	
	(** coq titles pretty_printing *)
	let coq_title (level : int) (title : string) : string list =
		(Xlist.tabulate (3 - level) (fun _ -> "")) @
		("  (** " ^ (Xstring.make level "*") ^ " " ^ title ^ " *)") ::
		[]
	
	let rec tactics_aggregate l =
		match l with
				[] -> []
			| (pat, _) :: l' ->
					let all_pat_assoc = Xlist.all_assoc pat l in
					let new_l' = Xlist.remove_all_assoc pat l' in
					let (body, post_body) = List.split all_pat_assoc in
					let consolidated_body =
						List.filter (fun l -> l <> []) (body @ post_body) in
					let composed_body =
						let revl = List.rev consolidated_body in
						let newl =
							match revl with
									[] -> []
								| h :: l ->
										h :: List.map (Xlist.map_last (fun s -> s ^" ||")) l in
						List.rev newl in
					let final_body = List.concat composed_body in
					(pat, final_body) :: tactics_aggregate new_l'
	
	let rec tactics_merge l =
		match l with
				[] -> []
			| (goal, body) :: l ->
					("        | [ |- context [ " ^ goal ^ " ] ] =>") ::
					(Xstring.replaces (("#inH#", "") :: []) [] body) @
					("        | [ H : context [ " ^ goal ^ " ] |- _ ] =>") ::
					(Xstring.replaces (("#inH#", " in H") :: []) [] body) @
					(tactics_merge l)
	
end
