module Xstring = struct
	
	(**
	* match a string with a list of string substitution and
	* returns the modified string.
	*
	* @params subst s
	* @param subst the list of word / replace.
	* @param s the string to modify.
	*
	* @return s with subst performed.
	*)
	let replace subst s =
		(* build the regexp * string from subst *)
		let replace_list =
			List.map (fun (s1, s2) -> (Str.regexp s1, s2)) subst in
		List.fold_left
			(fun ss -> fun (r, w) -> Str.global_replace r w ss)
			s replace_list
	;;
	
	(**
	* match a string list with a list of string substitution and
	* a list of line substitution returns the modified string.
	*
	* @params l_subst s_subst sl
	* @param l_subst the list of line_word / replace.
	* @param s_subst the list of word / replace.
	* @param sl the string list to modify.
	*
	* @return sl with firts l_subst and then s_subst performed.
	*)
	let replaces s_subst l_subst sl =
		let replace_line (s', sl') sl =
			List.fold_right
				(fun s -> fun sl -> if s = s' then sl' @ sl else s :: sl) sl []
		in
		let sl_l_subst =
			List.fold_left
				(fun sl -> fun repl -> replace_line repl sl) sl l_subst
		in
		List.map (fun s -> replace s_subst s) sl_l_subst
	;;
	
	(**
	* make a string by repetition of a pattern.
	*
	* @params nb pattern
	* @param nb the number of repetition.
	* @param pattern the the string pattern to repeat.
	*
	* @return the string resulting from the concatenation of nb pattern.
	*)
	let rec make nb pattern =
		if nb <= 0 then ""
		else pattern ^ (make (nb - 1) pattern)
	;;
	
	(**
	* insert tabulation before a string.
	*
	* @params n s
	* @param n the number of tabulation (of 2 spaces).
	* @param s the string.
	*
	* @return the string resulting from the concatenation of n "  " before s.
	*)
	let rec tabulate n s =
		if n <= 0 then s
		else "  " ^ tabulate (n - 1) s
	
	
end
